unit QNConfig;
interface

type str4=string[4];
     str2=string[2];

const qncsMagic='Quik';
      qncsMagic2=$65746F4E;
      qncsWinPosX='LWPX';
      qncsWinPosY='LWPY';
      qncsWinSizeX='LWSX';
      qncsWinSizeY='LWSY';
      qncsShade='Shad';
      qncsOnTop='@Top';
      qncsSnapArea='Snap';
      qncsBlend='blnd';
      qncsBlendValue='bVal';
      qncsDefaultFixed='DefF';
      qncsDefaultWrap='DefW';
      qncsVarFontName='VFfn';
      qncsVarFontSize='VFsz';
      qncsVarFontStyle='VFst';
      qncsFixFontName='FFfn';
      qncsFixFontSize='FFsz';
      qncsFixFontStyle='FFst';
      qncsTextFGColor='TFGC';
      qncsTextBGColor='TBGC';
      qncsWinBGColor='WBGC';
      qncsTabCount='Tabs';
      qncstName='tn';
      qncstText='tt';
      qncstSelStart='ss';
      qncstSelLength='sl';
      qncstFixed='mf';
      qncstWrap='mw';
      qncsActiveTab='aTab';
      csTerminate=#0#0#0#0;

type TQNConfigWriter=object
                       Data:array of integer;
                       Strings:array of byte;
                       procedure Init;
                       procedure PutInt(const ID:str4;Value:integer);
                       procedure PutBool(const ID:str4;Value:boolean);
                       procedure PutStr(const ID:str4;const Value:string);
                       procedure Terminate;
                       function SaveToFile(const FileName:string):boolean;
                       procedure Done;
                     end;

type TQNConfigReader=object
                       Data:array of byte;
                       StringBase:integer;
                       ReadPos:integer;
                       ID:str4; Value:integer;
                       ShortID:str2; SubID:integer;
                       function Init(const FileName:string):boolean;
                       procedure GetNextToken;
                       function GetStr:string;
                       procedure Done;
                     end;

function ComposeFourCC(const ShortID:str2;No:integer):str4;

implementation

function ComposeFourCC(const ShortID:str2;No:integer):str4;
begin
  Result[0]:=#4;
  Result[1]:=ShortID[1];
  Result[2]:=ShortID[2];
  Result[3]:=char(No SHR 8);
  Result[4]:=char(No AND $FF);
end;

procedure TQNConfigWriter.Init;
begin
  SetLength(Data,0);
  SetLength(Strings,0);
end;

procedure TQNConfigWriter.Done;
begin
  Finalize(Strings);
  Finalize(Data);
end;

procedure TQNConfigWriter.PutInt(const ID:str4;Value:integer);
begin
  SetLength(Data,High(Data)+3);
  move(ID[1],Data[High(Data)-1],4);
  Data[High(Data)]:=Value;
end;

procedure TQNConfigWriter.PutBool(const ID:str4;Value:boolean);
begin
  PutInt(ID,byte(Value));
end;

procedure TQNConfigWriter.PutStr(const ID:str4;const Value:string);
var StringPos,i:integer;
begin
  StringPos:=High(Strings)+1;
  SetLength(Data,High(Data)+3);
  move(ID[1],Data[High(Data)-1],4);
  Data[High(Data)]:=-4-StringPos;
  SetLength(Strings,StringPos+1+length(Value));
  move(Value[1],Strings[StringPos],length(Value));
  Strings[StringPos+length(Value)]:=0;
  for i:=StringPos to (StringPos+length(Value)-1) do
    if byte(Strings[i]-1)<32 then
      Strings[i]:=byte(byte(Strings[i]-1) XOR 31)+1;
end;

procedure TQNConfigWriter.Terminate;
begin
  SetLength(Data,High(Data)+2);
  Data[High(Data)]:=0;
end;

function TQNConfigWriter.SaveToFile(const FileName:string):boolean;
var f:file;
begin
  AssignFile(f,FileName);
  {$I-} Rewrite(f,1); {$I+}
  Result:=(IOResult=0);
  if not Result then exit;
  BlockWrite(f,Data[0],(High(Data)+1) SHL 2);
  BlockWrite(f,Strings[0],High(Strings)+1);
  CloseFile(f);
end;

function TQNConfigReader.Init(const FileName:string):boolean;
var f:file; i:integer;
begin
  AssignFile(f,FileName);
  {$I-} Reset(f,1); {$I+}
  Result:=(IOResult=0);
  if not Result then exit;
  SetLength(Data,FileSize(f));
  BlockRead(f,Data[0],FileSize(f));
  CloseFile(f);
  StringBase:=0;
  while (StringBase<High(Data)) AND (Data[StringBase]<>0) do inc(StringBase,8);
  if StringBase>(High(Data)-3) then begin Result:=false; exit; end;
  for i:=(StringBase+4) to High(Data) do
    if byte(Data[i]-1)<32 then
      Data[i]:=byte(byte(Data[i]-1) XOR 31)+1;
  ReadPos:=0; GetNextToken;
end;

procedure TQNConfigReader.GetNextToken;
begin
  ID[0]:=#4;
  move(Data[ReadPos],ID[1],4);
  move(Data[ReadPos+4],Value,4);
  inc(ReadPos,8);
  ShortID[0]:=#2;
  ShortID[1]:=ID[1];
  ShortID[2]:=ID[2];
  SubID:=(integer(byte(ID[3])) SHL 8) OR integer(byte(ID[4]));
end;

function TQNConfigReader.GetStr:string;
var sStart,sEnd:integer;
begin
  if Value>-4 then begin Result:=''; exit; end;
  sStart:=StringBase-Value; sEnd:=sStart;
  while (sEnd<High(Data)) AND (Data[sEnd]<>0) do inc(sEnd);
  SetLength(Result,sEnd-sStart);
  move(Data[sStart],Result[1],sEnd-sStart);
end;

procedure TQNConfigReader.Done;
begin
  Finalize(Data);
end;

end.

