unit QNMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, Menus, ImgList;

const CONFIGFILE='QuikNote.cfg';

type TNoteTab=record
                Name:shortstring;
                Text:string;
                SelStart,SelLength:integer;
                Button:TSpeedButton;
                Fixed,Wrap:boolean;
              end;

type
  TQNMainForm = class(TForm)
    BNew: TSpeedButton;
    Core: TMemo;
    BOptions: TSpeedButton;
    Options: TPanel;
    BCloseOptions: TSpeedButton;
    BFixed: TSpeedButton;
    BWrap: TSpeedButton;
    TabMenu: TPopupMenu;
    MClose: TMenuItem;
    OPTN_Spacer1: TBevel;
    Bevel1: TBevel;
    BOnTop: TSpeedButton;
    MFixed: TMenuItem;
    MWrap: TMenuItem;
    N1: TMenuItem;
    N2: TMenuItem;
    MOpen: TMenuItem;
    MSave: TMenuItem;
    ImageList: TImageList;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    BSetup: TSpeedButton;
    MRename: TMenuItem;
    BSearch: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure BNewClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure BOptionsClick(Sender: TObject);
    procedure BCloseOptionsClick(Sender: TObject);
    procedure BWrapClick(Sender: TObject);
    procedure BFixedClick(Sender: TObject);
    procedure CoreKeyPress(Sender: TObject; var Key: Char);
    procedure TabMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure MCloseClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BOnTopClick(Sender: TObject);
    procedure CoreClick(Sender: TObject);
    procedure TabMenuPopup(Sender: TObject);
    procedure MFixedClick(Sender: TObject);
    procedure MWrapClick(Sender: TObject);
    procedure MOpenClick(Sender: TObject);
    procedure MSaveClick(Sender: TObject);
    procedure BSetupClick(Sender: TObject);
    procedure MRenameClick(Sender: TObject);
    procedure BSearchClick(Sender: TObject);
  private
    { Private declarations }
    ActiveTab:integer;
    Tabs:array of TNoteTab;
    RestoreHeight:integer;
    ScreenWidth,ScreenHeight:integer;
    IsFixed,IsWrap:boolean;
    cfgFile:string;
    procedure OnNCDoubleClick(var msg:TMessage); message WM_NCLBUTTONDBLCLK;
    procedure OnMove(var msg:TMessage); message WM_MOVE;
    procedure SetShadeMode(ShadeMode:boolean);
    procedure CreateButton(TabNo:integer);
    procedure ArrangeButtons;
    procedure TabClick(Sender: TObject);
    procedure TabDblClick(Sender: TObject);
    procedure SwitchToTab(TabNo:integer);
    procedure SetFixed(Fixed:boolean);
    procedure SetWrap(Wrap:boolean);
    procedure SetOnTop(_ot:boolean);
    procedure LoadConfig;
  public
    Win_IsNT:boolean; Win_Version:integer;
    SnapArea:integer;
    OnTop:boolean;
    DefaultFixed,DefaultWrap:boolean;
    VarFont,FixFont:TFont;
    { Public declarations }
  end;

var
  QNMainForm: TQNMainForm;

implementation

uses QNSetup,QNConfig, QNSearch;

{$R *.dfm}

procedure TQNMainForm.FormCreate(Sender: TObject);
var Metrics:TNonClientMetrics;
    Version:TOSVersionInfo;
    InitialTab:integer;
begin
//AllocConsole;
  RestoreHeight:=0;
  ScreenWidth:=Screen.Width; ScreenHeight:=Screen.Height;

  Version.dwOSVersionInfoSize:=sizeof(Version);
  GetVersionEx(Version);
  Win_IsNT:=(Version.dwPlatformId=VER_PLATFORM_WIN32_NT);
  Win_Version:=Version.dwMajorVersion;

  Metrics.cbSize:=sizeof(Metrics);
  if SystemParametersInfo(SPI_GETNONCLIENTMETRICS,sizeof(Metrics),@Metrics,0) then
    with Metrics.lfMenuFont do begin
      Font.Height:=lfHeight;
      Font.Name:=lfFaceName;
    end;

  SnapArea:=10;
  OnTop:=true;
  DefaultFixed:=false; DefaultWrap:=true;
  VarFont:=TFont.Create; with VarFont do begin Name:='Tahoma'; Size:=9; Style:=[]; end;
  FixFont:=TFont.Create; with FixFont do begin Name:='Lucida Console'; Size:=9; Style:=[]; end;
  SetLength(Tabs,0);

  cfgFile:=IncludeTrailingPathDelimiter(GetCurrentDir)+CONFIGFILE;
  LoadConfig; InitialTab:=ActiveTab;

  IsFixed:=not(DefaultFixed); SetFixed(DefaultFixed);
  IsWrap:=not(DefaultWrap); SetWrap(DefaultWrap);
  OnTop:=not(OnTop); SetOnTop(not(OnTop));

  ActiveTab:=-1;
  if High(Tabs)<0 then BNewClick(nil) else SwitchToTab(InitialTab);
end;

procedure TQNMainForm.FormDestroy(Sender: TObject);
var i:integer;
    cfg:TQNConfigWriter;
    LastActiveTab:integer;
  function Style2Int(st:TFontStyles):integer;
  begin
    Result:=0;
    if (fsBold IN st) then Result:=Result OR 1;
    if (fsItalic IN st) then Result:=Result OR 2;
  end;
begin
  LastActiveTab:=ActiveTab;
  SwitchToTab(-1);
  with cfg do begin
    Init;
    PutInt(qncsMagic,qncsMagic2);
    PutInt(qncsWinPosX,Left);
    PutInt(qncsWinPosY,Top);
    PutInt(qncsWinSizeX,Width);
    if RestoreHeight=0 then begin
      PutInt(qncsWinSizeY,Height);
      PutInt(qncsShade,0);
    end else begin
      PutInt(qncsWinSizeY,RestoreHeight);
      PutInt(qncsShade,1);
    end;
    PutBool(qncsOnTop,OnTop);
    PutInt(qncsSnapArea,SnapArea);
    PutBool(qncsBlend,AlphaBlend);
    PutInt(qncsBlendValue,AlphaBlendValue);
    PutBool(qncsDefaultFixed,DefaultFixed);
    PutBool(qncsDefaultWrap,DefaultWrap);
    PutStr(qncsVarFontName,VarFont.Name);
    PutInt(qncsVarFontSize,VarFont.Size);
    PutInt(qncsVarFontStyle,Style2Int(VarFont.Style));
    PutStr(qncsFixFontName,FixFont.Name);
    PutInt(qncsFixFontSize,FixFont.Size);
    PutInt(qncsFixFontStyle,Style2Int(FixFont.Style));
    PutInt(qncsTextFGColor,FixFont.Color);
    PutInt(qncsTextBGColor,Core.Color);
    PutInt(qncsWinBGColor,Color);
    PutInt(qncsTabCount,High(Tabs)+1);
    for i:=0 to High(Tabs) do with Tabs[i] do begin
      PutStr(ComposeFourCC(qncstName,i),Name);
      PutStr(ComposeFourCC(qncstText,i),Text);
      PutInt(ComposeFourCC(qncstSelStart,i),SelStart);
      PutInt(ComposeFourCC(qncstSelLength,i),SelLength);
      PutBool(ComposeFourCC(qncstFixed,i),Fixed);
      PutBool(ComposeFourCC(qncstWrap,i),Wrap);
    end;
    PutInt(qncsActiveTab,LastActiveTab);
    Terminate;
    SaveToFile(cfgFile);
    Done;
  end;
  for i:=0 to High(Tabs) do Tabs[i].Button.Free;
end;

procedure TQNMainForm.LoadConfig;
var i:integer;
    cfg:TQNConfigReader;
  function Int2Style(i:integer):TFontStyles;
  begin
    Result:=[];
    if (i AND 1<>0) then Result:=Result+[fsBold];
    if (i AND 2<>0) then Result:=Result+[fsItalic];
  end;
begin
  with cfg do begin
    if not Init(cfgFile) then exit;
    if (ID<>qncsMagic) OR (Value<>qncsMagic2) then exit;
    GetNextToken; 
    while ID<>csTerminate do begin
      if ID=qncsWinPosX then Left:=Value;
      if ID=qncsWinPosY then Top:=Value;
      if ID=qncsWinSizeX then Width:=Value;
      if ID=qncsWinSizeY then Height:=Value;
      if (ID=qncsShade) AND (Value<>0) then SetShadeMode(true);
      if ID=qncsOnTop then OnTop:=(Value<>0);
      if ID=qncsSnapArea then SnapArea:=Value;
      if ID=qncsBlend then AlphaBlend:=(Value<>0);
      if ID=qncsBlendValue then AlphaBlendValue:=Value;
      if ID=qncsDefaultFixed then DefaultFixed:=(Value<>0);
      if ID=qncsDefaultWrap then DefaultWrap:=(Value<>0);
      if ID=qncsVarFontName then VarFont.Name:=GetStr;
      if ID=qncsVarFontSize then VarFont.Size:=Value;
      if ID=qncsVarFontStyle then VarFont.Style:=Int2Style(Value);
      if ID=qncsFixFontName then FixFont.Name:=GetStr;
      if ID=qncsFixFontSize then FixFont.Size:=Value;
      if ID=qncsFixFontStyle then FixFont.Style:=Int2Style(Value);
      if ID=qncsTextFGColor then begin VarFont.Color:=Value; FixFont.Color:=Value; end;
      if ID=qncsTextBGColor then Core.Color:=Value;
      if ID=qncsWinBGColor then Color:=Value;
      if ID=qncsTabCount then begin
        SetLength(Tabs,Value);
        for i:=0 to High(Tabs) do with Tabs[i] do begin
          Name:='NAME_LOST'; Text:='';
          SelStart:=0; SelLength:=0;
          Fixed:=DefaultFixed; Wrap:=DefaultWrap;
        end;
      end;
      if (SubID>=0) AND (SubID<=High(Tabs)) then with Tabs[SubID] do begin
        if ShortID=qncstName then Name:=GetStr;
        if ShortID=qncstText then Text:=GetStr;
        if ShortID=qncstSelStart then SelStart:=Value;
        if ShortID=qncstSelLength then SelLength:=Value;
        if ShortID=qncstFixed then Fixed:=(Value<>0);
        if ShortID=qncstWrap then Wrap:=(Value<>0);
      end;
      if ID=qncsActiveTab then ActiveTab:=Value;
      GetNextToken;
    end;
    Done;
  end;
  for i:=0 to High(Tabs) do CreateButton(i);
  ArrangeButtons;
end;

procedure TQNMainForm.OnNCDoubleClick(var msg:TMessage);
begin
  if WindowState=wsMaximized then begin
    WindowState:=wsNormal;
    exit;
  end;
  SetShadeMode(RestoreHeight=0);
  SetOnTop(OnTop);
  msg.Result:=0;
end;

procedure TQNMainForm.SetShadeMode(ShadeMode:boolean);
begin
  if ShadeMode then begin
    RestoreHeight:=Height;
    Constraints.MinHeight:=0;
    ClientHeight:=0;
    Constraints.MinHeight:=Height;
    Constraints.MaxHeight:=Height;
    BorderIcons:=[biSystemMenu];
  end else begin
    BorderIcons:=[biSystemMenu,biMinimize,biMaximize];
    Constraints.MaxHeight:=4096;
    Height:=RestoreHeight;
    Constraints.MinHeight:=99;
    RestoreHeight:=0;
    Core.Width:=ClientWidth-24;
    Core.Height:=ClientHeight-24;
    BSetup.Top:=ClientHeight-24;
  end;
end;

procedure TQNMainForm.OnMove(var msg:TMessage);
var i:integer;
begin
  msg.Result:=0;
  if abs(Left)<=SnapArea then begin Left:=0; exit; end;
  if abs(Top)<=SnapArea then begin Top:=0; exit; end;
  i:=ScreenWidth-Width;
  if abs(i-Left)<=SnapArea then begin Left:=i; exit; end;
  i:=ScreenHeight-Height;
  if abs(i-Top)<=SnapArea then begin Top:=i; exit; end;
end;                  

procedure TQNMainForm.BNewClick(Sender: TObject);
var i,tres,tval,HighestNumber:integer;
begin
  HighestNumber:=0;
  for i:=0 to High(Tabs) do begin
    with Tabs[i] do begin
      if copy(Name,1,3)='Neu' then begin
        val(copy(Name,4,10),tval,tres);
        if (tres=0) AND (tval>HighestNumber) then
          HighestNumber:=tval;
      end;
    end;
  end;
  SetLength(Tabs,High(Tabs)+2);
  with Tabs[High(Tabs)] do begin
    Name:='Neu'+IntToStr(HighestNumber+1);
    Text:=''; SelStart:=0; SelLength:=0;
    Fixed:=DefaultFixed; Wrap:=DefaultWrap;
  end;
  CreateButton(High(Tabs));
  ArrangeButtons;
  SwitchToTab(High(Tabs));
end;

procedure TQNMainForm.CreateButton(TabNo:integer);
begin
  with Tabs[TabNo] do begin
    Button:=TSpeedButton.Create(Self);
    with Button do begin
      Parent:=Self;
      Flat:=true;
      Top:=0; Height:=24;
      Caption:=Tabs[TabNo].Name;
      Hint:=Tabs[TabNo].Name;
      ShowHint:=true;
      GroupIndex:=1;
      Tag:=TabNo;
      OnClick:=TabClick;
      OnDblClick:=TabDblClick;
      OnMouseDown:=TabMouseDown;
      PopupMenu:=TabMenu;
    end;
  end;
end;

procedure TQNMainForm.ArrangeButtons;
var i,CurrentX,BWidth:integer;
begin
  if High(Tabs)<0 then exit;
  BWidth:=(ClientWidth-24) DIV (High(Tabs)+1);
  CurrentX:=24;
  for i:=0 to High(Tabs) do
    with Tabs[i].Button do begin
      Left:=CurrentX;
      Width:=BWidth;
      inc(CurrentX,BWidth);
    end;
end;

procedure TQNMainForm.FormResize(Sender: TObject);
begin
  ArrangeButtons;
end;

procedure TQNMainForm.TabClick(Sender: TObject);
begin
  SwitchToTab((Sender as TSpeedButton).Tag);
end;

procedure TQNMainForm.TabDblClick(Sender: TObject);
begin
  with Tabs[(Sender as TSpeedButton).Tag] do begin
    Name:=InputBox('QuikNote: Notiz umbenennen','Wie wollen Sie die Notiz nennen?',Name);
    Button.Caption:=Name;
    Button.Hint:=Name;
  end;
end;

procedure TQNMainForm.SwitchToTab(TabNo:integer);
begin
  if (TabNo=ActiveTab) then exit;
  if ActiveTab>=0 then with Tabs[ActiveTab] do begin
    Text:=Core.Text;
    SelStart:=Core.SelStart;
    SelLength:=Core.SelLength;
    Fixed:=IsFixed; Wrap:=IsWrap;
  end;
  if (TabNo<0) OR (TabNo>High(Tabs)) then exit;
  with Tabs[TabNo] do begin
    Core.Text:=Text;
    SetFixed(Fixed); SetWrap(Wrap);
    Core.SelStart:=SelStart;
    Core.SelLength:=SelLength;
    Button.Down:=true;
  end;
  ActiveTab:=TabNo;
end;

procedure TQNMainForm.BOptionsClick(Sender: TObject);
begin
  Options.Visible:=true;
end;

procedure TQNMainForm.BCloseOptionsClick(Sender: TObject);
begin
  Options.Visible:=false;
end;

procedure TQNMainForm.SetWrap(Wrap:boolean);
begin
  if Wrap<>IsWrap then begin
    if Wrap then begin
      Core.WordWrap:=true;
      Core.ScrollBars:=ssVertical;
    end else begin
      Core.WordWrap:=false;
      Core.ScrollBars:=ssBoth;
    end;
  end;
  IsWrap:=Wrap;
  BWrap.Down:=Wrap;
end;

procedure TQNMainForm.SetFixed(Fixed:boolean);
begin
  if Fixed<>IsFixed then begin
    if Fixed then Core.Font:=FixFont else Core.Font:=VarFont;
  end;
  IsFixed:=Fixed;
  BFixed.Down:=Fixed;
end;

procedure TQNMainForm.BWrapClick(Sender: TObject);
begin
  SetWrap(BWrap.Down);
  Options.Visible:=false;
end;

procedure TQNMainForm.BFixedClick(Sender: TObject);
begin
  SetFixed(BFixed.Down);
  Options.Visible:=false;
end;

procedure TQNMainForm.CoreKeyPress(Sender: TObject; var Key: Char);
begin
  if Key=^A then with Core do begin
    SelStart:=0; SelLength:=length(Text);
    Key:=#0;
  end;
end;

procedure TQNMainForm.TabMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  SwitchToTab((Sender as TSpeedButton).Tag);
end;

procedure TQNMainForm.MCloseClick(Sender: TObject);
var TabNo,i:integer;
begin
  TabNo:=ActiveTab;
  Tabs[TabNo].Button.Free;
  for i:=(TabNo+1) to High(Tabs) do begin
    Tabs[i].Button.Tag:=i-1;
    Tabs[i-1]:=Tabs[i];
  end;
  SetLength(Tabs,High(Tabs));
  ActiveTab:=-1;
  if High(Tabs)>=0 then begin
    if TabNo>High(Tabs) then TabNo:=High(Tabs);
    SwitchToTab(TabNo);
    ArrangeButtons;
  end else BNewClick(nil);
end;

procedure TQNMainForm.SetOnTop(_ot:boolean);
var Par:cardinal;
begin
  OnTop:=_ot;
  if OnTop then Par:=HWND_TOPMOST else Par:=HWND_NOTOPMOST;
  SetWindowPos(Handle,Par,0,0,0,0,SWP_NOMOVE OR SWP_NOSIZE);
  if Assigned(QNSearchForm) AND (QNSearchForm.Visible) then
    SetWindowPos(QNSearchForm.Handle,Par,0,0,0,0,SWP_NOMOVE OR SWP_NOSIZE);
  BOnTop.Down:=OnTop;
end;

procedure TQNMainForm.BOnTopClick(Sender: TObject);
begin
  SetOnTop(not(OnTop));
  Options.Visible:=false;
end;

procedure TQNMainForm.CoreClick(Sender: TObject);
begin
  Options.Visible:=false;
end;

procedure TQNMainForm.TabMenuPopup(Sender: TObject);
begin
  MFixed.Checked:=IsFixed;
  MWrap.Checked:=IsWrap;
end;

procedure TQNMainForm.MFixedClick(Sender: TObject);
begin
  SetFixed(not(MFixed.Checked));
end;

procedure TQNMainForm.MWrapClick(Sender: TObject);
begin
  SetWrap(not(MWrap.Checked));
end;

procedure TQNMainForm.MOpenClick(Sender: TObject);
var f:file; s:string; i:integer;
begin
  if not OpenDialog.Execute then exit;
  AssignFile(f,OpenDialog.FileName); Reset(f,1);
  i:=FileSize(f); SetLength(s,i);
  BlockRead(f,s[1],i);
  CloseFile(f);
  Core.Text:=s;
end;

procedure TQNMainForm.MSaveClick(Sender: TObject);
var f:file; s:string;
begin
  if not SaveDialog.Execute then exit;
  s:=Core.Text;
  AssignFile(f,SaveDialog.FileName); Rewrite(f,1);
  BlockWrite(f,s[1],length(s));
  CloseFile(f);
end;

procedure TQNMainForm.BSetupClick(Sender: TObject);
begin
  if QNSetupForm.ShowModal<>mrOK then exit;
  IsFixed:=not(IsFixed); SetFixed(not(IsFixed));
  SetOnTop(OnTop);
end;

procedure TQNMainForm.MRenameClick(Sender: TObject);
begin
  TabDblClick(Tabs[ActiveTab].Button);
end;

procedure TQNMainForm.BSearchClick(Sender: TObject);
begin
  if BSearch.Down then begin
    QNSearchForm.Show;
    SetOnTop(OnTop);
  end else begin
    QNSearchForm.Hide;
  end;
end;

end.
