unit QNSetup;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls;

type
  TQNSetupForm = class(TForm)
    BOK: TButton;
    BCancel: TButton;
    GB_General: TGroupBox;
    CB_OnTop: TCheckBox;
    GB_Defaults: TGroupBox;
    CB_DefaultFixed: TCheckBox;
    CB_DefaultWrap: TCheckBox;
    LVarFont: TLabel;
    PVarFont: TPanel;
    BVarFont: TButton;
    PFixFont: TPanel;
    BFixFont: TButton;
    LFixFont: TLabel;
    FontDialog: TFontDialog;
    LTextFGColor: TLabel;
    LTextBGColor: TLabel;
    LWinBGColor: TLabel;
    PTextFGColor: TPanel;
    BTextFGColor: TButton;
    PTextBGColor: TPanel;
    BTextBGColor: TButton;
    PWinBGColor: TPanel;
    BWinBGColor: TButton;
    CB_DefaultTextFGColor: TCheckBox;
    CB_DefaultTextBGColor: TCheckBox;
    CB_DefaultWinBGColor: TCheckBox;
    ColorDialog: TColorDialog;
    LSnapArea: TLabel;
    CSnapArea: TTrackBar;
    LSnapAreaValue: TLabel;
    CB_Blend: TCheckBox;
    CBlendValue: TTrackBar;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure BFontClick(Sender: TObject);
    procedure CB_DefaultTextFGColorClick(Sender: TObject);
    procedure BTextFGColorClick(Sender: TObject);
    procedure CB_DefaultTextBGColorClick(Sender: TObject);
    procedure BTextBGColorClick(Sender: TObject);
    procedure CB_DefaultWinBGColorClick(Sender: TObject);
    procedure BWinBGColorClick(Sender: TObject);
    procedure BOKClick(Sender: TObject);
    procedure CSnapAreaChange(Sender: TObject);
    procedure CB_BlendClick(Sender: TObject);
  private
    { Private declarations }
    CanBlend:boolean;
  public
    { Public declarations }
  end;

function FontDescription(const Font:TFont):string;

var
  QNSetupForm: TQNSetupForm;

implementation

uses QNMain;

{$R *.dfm}

procedure TQNSetupForm.FormCreate(Sender: TObject);
begin
  Font:=QNMainForm.Font;
  BVarFont.Font:=Font;
  BFixFont.Font:=Font;
  CanBlend:=QNMainForm.Win_IsNT AND (QNMainForm.Win_Version>=5);
  CB_Blend.Enabled:=CanBlend;
end;

procedure TQNSetupForm.FormShow(Sender: TObject);
begin
  with QNMainForm do begin
    CB_OnTop.Checked:=OnTop;
    CB_DefaultFixed.Checked:=DefaultFixed;
    CB_DefaultWrap.Checked:=DefaultWrap;
    PVarFont.Font:=VarFont; PVarFont.Caption:=FontDescription(VarFont);
    PFixFont.Font:=FixFont; PFixFont.Caption:=FontDescription(FixFont);
    PTextFGColor.Color:=FixFont.Color;
    CB_DefaultTextFGColor.Checked:=(FixFont.Color=clWindowText);
    PTextBGColor.Color:=Core.Color;
    CB_DefaultTextBGColor.Checked:=(Core.Color=clWindow);
    PVarFont.Color:=Core.Color; PFixFont.Color:=Core.Color;
    PWinBGColor.Color:=Color;
    CB_DefaultWinBGColor.Checked:=(Color=clBtnFace);
    CSnapArea.Position:=SnapArea;
    CB_Blend.Checked:=AlphaBlend;
    CBlendValue.Position:=AlphaBlendValue;
    CB_BlendClick(nil);
  end;
end;

function FontDescription(const Font:TFont):string;
begin
  with Font do begin
    Result:=Name;
    if (fsBold IN Style) then Result:=Result+' Fett';
    if (fsItalic IN Style) then Result:=Result+' Kursiv';
    Result:=Result+' '+IntToStr(Size)+' pt';
  end;
end;


procedure TQNSetupForm.BFontClick(Sender: TObject);
var Panel:TPanel;
begin
  Panel:=((Sender as TButton).Parent) as TPanel;
  FontDialog.Font:=Panel.Font;
  if (Sender as TButton).Tag=1
    then FontDialog.Options:=FontDialog.Options-[fdFixedPitchOnly]
    else FontDialog.Options:=FontDialog.Options+[fdFixedPitchOnly];
  if not FontDialog.Execute then exit;
  FontDialog.Font.Color:=PTextFGColor.Color;
  Panel.Font:=FontDialog.Font;
  Panel.Caption:=FontDescription(Panel.Font);
end;

procedure TQNSetupForm.CB_DefaultTextFGColorClick(Sender: TObject);
begin
  if not CB_DefaultTextFGColor.Checked then exit;
  PTextFGColor.Color:=clWindowText;
  PVarFont.Font.Color:=clWindowText;
  PFixFont.Font.Color:=clWindowText;
end;

procedure TQNSetupForm.BTextFGColorClick(Sender: TObject);
begin
  ColorDialog.Color:=PTextFGColor.Color;
  if not ColorDialog.Execute then exit;
  CB_DefaultTextFGColor.Checked:=false;
  PTextFGColor.Color:=ColorDialog.Color;
  PVarFont.Font.Color:=ColorDialog.Color;
  PFixFont.Font.Color:=ColorDialog.Color;
end;

procedure TQNSetupForm.CB_DefaultTextBGColorClick(Sender: TObject);
begin
  if not CB_DefaultTextBGColor.Checked then exit;
  PTextBGColor.Color:=clWindow;
  PVarFont.Color:=clWindow;
  PFixFont.Color:=clWindow;
end;

procedure TQNSetupForm.BTextBGColorClick(Sender: TObject);
begin
  ColorDialog.Color:=PTextBGColor.Color;
  if not ColorDialog.Execute then exit;
  CB_DefaultTextBGColor.Checked:=false;
  PTextBGColor.Color:=ColorDialog.Color;
  PVarFont.Color:=ColorDialog.Color;
  PFixFont.Color:=ColorDialog.Color;
end;

procedure TQNSetupForm.CB_DefaultWinBGColorClick(Sender: TObject);
begin
  if not CB_DefaultWinBGColor.Checked then exit;
  PWinBGColor.Color:=clBtnFace;
end;

procedure TQNSetupForm.BWinBGColorClick(Sender: TObject);
begin
  ColorDialog.Color:=PWinBGColor.Color;
  if not ColorDialog.Execute then exit;
  CB_DefaultWinBGColor.Checked:=false;
  PWinBGColor.Color:=ColorDialog.Color;
end;

procedure TQNSetupForm.BOKClick(Sender: TObject);
begin
  with QNMainForm do begin
    VarFont:=PVarFont.Font;
    FixFont:=PFixFont.Font;
    OnTop:=CB_OnTop.Checked;
    DefaultFixed:=CB_DefaultFixed.Checked;
    DefaultWrap:=CB_DefaultWrap.Checked;
    Core.Color:=PTextBGColor.Color;
    Color:=PWinBGColor.Color;
    SnapArea:=CSnapArea.Position;
    AlphaBlend:=CB_Blend.Checked;
    AlphaBlendValue:=CBlendValue.Position;
  end;
end;

procedure TQNSetupForm.CSnapAreaChange(Sender: TObject);
begin
  LSnapAreaValue.Caption:=IntToStr(CSnapArea.Position)+' Pixel';
end;

procedure TQNSetupForm.CB_BlendClick(Sender: TObject);
begin
  CBlendValue.Enabled:=CanBlend AND CB_Blend.Checked;
end;

end.
